function [e, time] = Cayley(R)
%
% This function takes as input 'R', a proper rotation matrix, and returns 
% the corresponding Euler parameters 'e' using Cayley factorization method 
% adapted to the three dimensional case. For details see: 
% 
% F. Thomas, "Approaching dual quaternions from matrix algebra," 
% IEEE Transactions on Robotics, Vol. 30, No. 5, pp. 1037-1048, 2014.
%
% S. Srabandi and F. thomas, "Computing Quaternions from Rotation Matrices and 
% its Application to the Orthonormalization of Noisy Rotation Matrices,"
% submitted for publication, 2019.
%

tic;

e(1) = 0.25*sqrt((1+R(1,1)+R(2,2)+R(3,3))^2 + (R(3,2)-R(2,3))^2 + (R(1,3)-R(3,1))^2 + (R(2,1)-R(1,2))^2); 
e(2) = 0.25*sqrt((R(3,2)-R(2,3))^2 + (1+R(1,1)-R(2,2)-R(3,3))^2 + (R(1,2)+R(2,1))^2 + (R(3,1)+R(1,3))^2);
e(3) = 0.25*sqrt((R(1,3)-R(3,1))^2 + (R(1,2)+R(2,1))^2 + (1-R(1,1)+R(2,2)-R(3,3))^2 + (R(2,3)+R(3,2))^2);
e(4) = 0.25*sqrt((R(2,1)-R(1,2))^2 + (R(3,1)+R(1,3))^2 + (R(2,3)+R(3,2))^2 + (1-R(1,1)-R(2,2)+R(3,3))^2);

[~, index] = max(e);

switch index
    case 1
        e(1) =                     e(1);
        e(2) = sign(R(3,2)-R(2,3))*e(2);
        e(3) = sign(R(1,3)-R(3,1))*e(3);
        e(4) = sign(R(2,1)-R(1,2))*e(4);
    case 2
        e(1) = sign(R(3,2)-R(2,3))*e(1);
        e(2) =                     e(2);
        e(3) = sign(R(2,1)+R(1,2))*e(3);
        e(4) = sign(R(1,3)+R(3,1))*e(4);
    case 3
        e(1) = sign(R(1,3)-R(3,1))*e(1);
        e(2) = sign(R(2,1)+R(1,2))*e(2);
        e(3) =                     e(3);
        e(4) = sign(R(3,2)+R(2,3))*e(4);
    case 4
        e(1) = sign(R(2,1)-R(1,2))*e(1);
        e(2) = sign(R(1,3)+R(3,1))*e(2);
        e(3) = sign(R(3,2)+R(2,3))*e(3);
        e(4) =                     e(4);
end

time=toc;
   
end

